/***************************************************************************
 * This file is part of the project AqFoxExt.                              *
 * AqFoxExt (c) by 2011 Martin Preuss, all rights reserved.                *
 *                                                                         *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FXX_CHECKTREE_HPP
#define FXX_CHECKTREE_HPP


#include <fx.h>

#include <aqfoxext/aqfoxext.h>


class FXXCheckTree;


class AQFOXEXT_API FXXCheckTreeItem: public FXFoldingItem {
protected:
  FXbool m_isChecked;
  FXbool m_useOwnColors;
  FXColor m_backColor;
  FXColor m_textColor;

  virtual void draw(const FXFoldingList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;

  virtual void drawColumn(const FXXCheckTree* list,
			  FXDC &dc,
                          FXint col,
			  FXint x, FXint y, FXint w, FXint h,
			  const FXchar* string, FXuint length) const;

  virtual void drawColumnRightJustified(const FXXCheckTree* list, FXDC &dc, FXint col,
                                        FXint xx, FXint y, FXint w, FXint h,
                                        const FXchar* string, FXuint length) const;


public:
  FXXCheckTreeItem(const FXString& text, FXIcon* oi=NULL, FXIcon* ci=NULL, void* ptr=NULL);
  ~FXXCheckTreeItem();

  FXbool isChecked() const { return m_isChecked;};
  void setChecked(FXbool b) { m_isChecked=b;};

  FXint hitItem(const FXFoldingList* list, FXint x, FXint y) const;

  FXbool getUseOwnColors() const { return m_useOwnColors;};
  void setUseOwnColors(FXbool b) { m_useOwnColors=b;};

  FXColor getBackColor() const { return m_backColor;};
  void setBackColor(FXColor c) { m_backColor=c;};

  FXColor getTextColor() const { return m_textColor;};
  void setTextColor(FXColor c) { m_textColor=c;};

  virtual FXString getItemText(const FXXCheckTree* list) const;

  virtual FXString getColumnTooltipText(FXint col);
};



class AQFOXEXT_API FXXCheckTree: public FXFoldingList {
  FXDECLARE(FXXCheckTree)
  friend class FXXCheckTreeItem;
public:
  /** Options to use with @ref FOLDINGLIST_NORMAL and others */
  enum {
    OPTIONS_CHECKMARKS=0x04000000
  };

  FXXCheckTree(FXComposite *p,
	       FXObject* tgt=NULL, FXSelector sel=0,
	       FXuint opts=FOLDINGLIST_NORMAL,
	       FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  FXXCheckTree();

  virtual ~FXXCheckTree();

  FXuint getOptions() const { return options;};

  FXbool getHeaderArrowDir(int c) const;
  void setHeaderArrowDir(int c, FXbool b);

  virtual void sortByColumn(int c, bool up);
  void sort();

  long onHeaderClicked(FXObject*, FXSelector, void* ptr);
  long onQueryTip(FXObject* sender,FXSelector sel,void* ptr);

  void appendHeader(const FXString& text,FXIcon *icon=NULL,FXint size=1);

  FXColor baseColor;    // Base color
  FXColor hiliteColor;  // Highlight color
  FXColor shadowColor;  // Shadow color
  FXColor borderColor;  // Border color

};



/* some macros for sorting */
#define FXX_CHECKTREE_SORT_FN_UP_TXT(macro_class, macro_itemtype, macro_getmemberfn,         \
                                     macro_membertype, macro_memberprefix, macro_name)       \
FXint macro_class::sortFunctionUp##macro_name(const FXFoldingItem *it1,                      \
                                              const FXFoldingItem *it2) {                    \
  const FXXCheckTreeItem *ctItem1=dynamic_cast<const FXXCheckTreeItem*>(it1);                \
  const FXXCheckTreeItem *ctItem2=dynamic_cast<const FXXCheckTreeItem*>(it2);                \
                                                                                             \
  if (ctItem1 && ctItem2) {                                                                  \
    const macro_itemtype *tti1=dynamic_cast<const macro_itemtype*>(it1);                     \
    const macro_itemtype *tti2=dynamic_cast<const macro_itemtype*>(it2);                     \
    const macro_membertype *ba1=tti1->macro_getmemberfn();                                   \
    const macro_membertype *ba2=tti2->macro_getmemberfn();                                   \
    const char *s1=macro_memberprefix##_Get##macro_name(ba1);                                \
    const char *s2=macro_memberprefix##_Get##macro_name(ba2);                                \
                                                                                             \
    /* both are data items */                                                                \
    if (s1 && s2)                                                                            \
      return strcasecmp(s1, s2);                                                             \
    else if (s1)                                                                             \
      return 1;                                                                              \
    else if (s2)                                                                             \
      return -1;                                                                             \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}




#define FXX_CHECKTREE_SORT_FN_UP_VALUE(macro_class, macro_itemtype, macro_getmemberfn,       \
                                       macro_membertype, macro_memberprefix, macro_name)     \
FXint macro_class::sortFunctionUp##macro_name(const FXFoldingItem *it1,                      \
                                              const FXFoldingItem *it2) {                    \
  const FXXCheckTreeItem *ctItem1=dynamic_cast<const FXXCheckTreeItem*>(it1);                \
  const FXXCheckTreeItem *ctItem2=dynamic_cast<const FXXCheckTreeItem*>(it2);                \
                                                                                             \
  if (ctItem1 && ctItem2) {                                                                  \
    const macro_itemtype *tti1=dynamic_cast<const macro_itemtype*>(it1);                     \
    const macro_itemtype *tti2=dynamic_cast<const macro_itemtype*>(it2);                     \
    const macro_membertype *ba1=tti1->macro_getmemberfn();                                   \
    const macro_membertype *ba2=tti2->macro_getmemberfn();                                   \
    const AQDB_VALUE *v1=macro_memberprefix##_Get##macro_name(ba1);                          \
    const AQDB_VALUE *v2=macro_memberprefix##_Get##macro_name(ba2);                          \
                                                                                             \
    /* both are data items */                                                                \
    if (v1 && v2)                                                                            \
      return AQDB_Value_Compare(v1, v2);                                                     \
    else if (v1)                                                                             \
      return 1;                                                                              \
    else if (v2)                                                                             \
      return -1;                                                                             \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}



#define FXX_CHECKTREE_SORT_FN_UP_DATE(macro_class, macro_itemtype, macro_getmemberfn,        \
                                      macro_membertype, macro_memberprefix, macro_name)      \
FXint macro_class::sortFunctionUp##macro_name(const FXFoldingItem *it1,                      \
                                              const FXFoldingItem *it2) {                    \
  const FXXCheckTreeItem *ctItem1=dynamic_cast<const FXXCheckTreeItem*>(it1);                \
  const FXXCheckTreeItem *ctItem2=dynamic_cast<const FXXCheckTreeItem*>(it2);                \
                                                                                             \
  if (ctItem1 && ctItem2) {                                                                  \
    const macro_itemtype *tti1=dynamic_cast<const macro_itemtype*>(it1);                     \
    const macro_itemtype *tti2=dynamic_cast<const macro_itemtype*>(it2);                     \
    const macro_membertype *ba1=tti1->macro_getmemberfn();                                   \
    const macro_membertype *ba2=tti2->macro_getmemberfn();                                   \
    const GWEN_DATE *t1=macro_memberprefix##_Get##macro_name(ba1);                           \
    const GWEN_DATE *t2=macro_memberprefix##_Get##macro_name(ba2);                           \
                                                                                             \
    /* both are data items */                                                                \
    if (t1 && t2) {                                                                          \
      int d=GWEN_Date_Diff(t1, t2);                                                          \
                                                                                             \
      if (d<0)                                                                               \
        return -1;                                                                           \
      else if (d>0)                                                                          \
        return 1;                                                                            \
    }                                                                                        \
    else if (t1)                                                                             \
      return 1;                                                                              \
    else if (t2)                                                                             \
      return -1;                                                                             \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}



#define FXX_CHECKTREE_SORT_FN_UP_TIME(macro_class, macro_itemtype, macro_getmemberfn,        \
                                          macro_membertype, macro_memberprefix, macro_name)  \
FXint macro_class::sortFunctionUp##macro_name(const FXFoldingItem *it1,                      \
                                              const FXFoldingItem *it2) {                    \
  const FXXCheckTreeItem *ctItem1=dynamic_cast<const FXXCheckTreeItem*>(it1);                \
  const FXXCheckTreeItem *ctItem2=dynamic_cast<const FXXCheckTreeItem*>(it2);                \
                                                                                             \
  if (ctItem1 && ctItem2) {                                                                  \
    const macro_itemtype *tti1=dynamic_cast<const macro_itemtype*>(it1);                     \
    const macro_itemtype *tti2=dynamic_cast<const macro_itemtype*>(it2);                     \
    const macro_membertype *ba1=tti1->macro_getmemberfn();                                   \
    const macro_membertype *ba2=tti2->macro_getmemberfn();                                   \
    const GWEN_TIME *t1=macro_memberprefix##_Get##macro_name(ba1);                           \
    const GWEN_TIME *t2=macro_memberprefix##_Get##macro_name(ba2);                           \
                                                                                             \
    /* both are data items */                                                                \
    if (t1 && t2) {                                                                          \
      int d=GWEN_Time_Diff(t1, t2);                                                          \
                                                                                             \
      if (d<0)                                                                               \
        return -1;                                                                           \
      else if (d>0)                                                                          \
        return 1;                                                                            \
    }                                                                                        \
    else if (t1)                                                                             \
      return 1;                                                                              \
    else if (t2)                                                                             \
      return -1;                                                                             \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}



#define FXX_CHECKTREE_SORT_FN_UP_INT(macro_class, macro_itemtype, macro_getmemberfn,         \
                                     macro_membertype, macro_memberprefix, macro_name)       \
FXint macro_class::sortFunctionUp##macro_name(const FXFoldingItem *it1,                      \
                                              const FXFoldingItem *it2) {                    \
  const FXXCheckTreeItem *ctItem1=dynamic_cast<const FXXCheckTreeItem*>(it1);                \
  const FXXCheckTreeItem *ctItem2=dynamic_cast<const FXXCheckTreeItem*>(it2);                \
                                                                                             \
  if (ctItem1 && ctItem2) {                                                                  \
    const macro_itemtype *tti1=dynamic_cast<const macro_itemtype*>(it1);                     \
    const macro_itemtype *tti2=dynamic_cast<const macro_itemtype*>(it2);                     \
    const macro_membertype *ba1=tti1->macro_getmemberfn();                                   \
    const macro_membertype *ba2=tti2->macro_getmemberfn();                                   \
    int v1=macro_memberprefix##_Get##macro_name(ba1);                                        \
    int v2=macro_memberprefix##_Get##macro_name(ba2);                                        \
                                                                                             \
    /* both are data items */                                                                \
    if (v2<v1)                                                                               \
      return 1;                                                                              \
    else if (v1<v2)                                                                          \
      return -1;                                                                             \
    else                                                                                     \
      return 0;                                                                              \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}





#define FXX_CHECKTREE_SORT_FN_DOWN(macro_class, macro_itemtype, macro_getmemberfn,           \
                                   macro_membertype, macro_memberprefix, macro_name)         \
FXint macro_class::sortFunctionDown##macro_name(const FXFoldingItem *it1,                    \
                                                const FXFoldingItem *it2){                   \
  FXint rv;                                                                                  \
                                                                                             \
  rv=sortFunctionUp##macro_name(it1, it2);                                                   \
  if (rv<0)                                                                                  \
    return 1;                                                                                \
  else if (rv>0)                                                                             \
    return -1;                                                                               \
  return 0;                                                                                  \
}





#endif
