/***************************************************************************
 * This file is part of the project AqFoxExt.                              *
 * AqFoxExt (c) by 2011 Martin Preuss, all rights reserved.                *
 *                                                                         *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "fxx_action_p.hpp"
#include "fxx_iconset.hpp"



FXXActionMenuCommand::FXXActionMenuCommand(FXXAction *act,
                                           FXMenuPane* p,
                                           const FXString& text, FXIcon* ic,
                                           FXObject* tgt, FXSelector sel,
                                           FXuint opts)
:FXMenuCommand(p, text, ic, tgt, sel, opts)
,action(act)
{

}



FXXActionMenuCommand::~FXXActionMenuCommand() {
  if (action)
    action->removeMenuCommand(this);
}



FXXActionMenuCheck::FXXActionMenuCheck(FXXAction *act,
                                       FXMenuPane* p,
                                       const FXString& text,
                                       FXObject* tgt, FXSelector sel,
                                       FXuint opts)
:FXMenuCheck(p, text, tgt, sel, opts)
,action(act)
{

}



FXXActionMenuCheck::~FXXActionMenuCheck() {
  if (action)
    action->removeMenuCheck(this);
}







FXXActionToolButton::FXXActionToolButton(FXXAction *act,
                                         FXToolBar* p,
                                         const FXString& text, FXIcon* ic,
                                         FXObject* tgt, FXSelector sel,
                                         FXuint opts)
:FXButton(p, text, ic, tgt, sel, opts)
,action(act)
{

}



FXXActionToolButton::~FXXActionToolButton() {
  if (action)
    action->removeToolButton(this);
}





FXXActionCheckButton::FXXActionCheckButton(FXXAction *act,
                                           FXToolBar* p,
                                           const FXString& text,
                                           FXObject* tgt, FXSelector sel,
                                           FXuint opts)
:FXCheckButton(p, text, tgt, sel, opts)
,action(act)
{

}



FXXActionCheckButton::~FXXActionCheckButton() {
  if (action)
    action->removeCheckButton(this);
}








FXXAction::FXXAction(const FXString &n,
                     const FXString &t,
                     FXXIconSet *is,
                     FXObject *tgt,
                     FXSelector sel)
:FXObject()
,name(n)
,text(t)
,iconSet(is)
,target(tgt)
,selector(sel)
,m_check(FALSE)
{

}



FXXAction::~FXXAction() {
  removeFromAll();
}



void FXXAction::enable() {
  FXint i;

  // enable in all menu commands
  for (i=0; i<commandList.no(); i++) {
    FXWindow *w=commandList[i];
    if (w)
      w->enable();
  }

  // enable in all menu checks
  for (i=0; i<menuCheckList.no(); i++) {
    FXWindow *w=menuCheckList[i];
    if (w)
      w->enable();
  }

  // enable in all tool buttons
  for (i=0; i<toolButtonList.no(); i++) {
    FXWindow *w=toolButtonList[i];
    if (w)
      w->enable();
  }

  // disable in all check buttons
  for (i=0; i<checkButtonList.no(); i++) {
    FXWindow *w=checkButtonList[i];
    if (w)
      w->enable();
  }

  // enable in all menu commands
  for (i=0; i<containerList.no(); i++) {
    FXXMenuContainer *w=containerList[i];
    if (w)
      w->enable();
  }
}



void FXXAction::disable() {
  FXint i;

  // disable in all menu commands
  for (i=0; i<commandList.no(); i++) {
    FXWindow *w=commandList[i];
    if (w)
      w->disable();
  }

  // disable in all menu checks
  for (i=0; i<menuCheckList.no(); i++) {
    FXWindow *w=menuCheckList[i];
    if (w)
      w->disable();
  }

  // disable in all tool buttons
  for (i=0; i<toolButtonList.no(); i++) {
    FXWindow *w=toolButtonList[i];
    if (w)
      w->disable();
  }

  // disable in all check buttons
  for (i=0; i<checkButtonList.no(); i++) {
    FXWindow *w=checkButtonList[i];
    if (w)
      w->disable();
  }

  // disable in all menu containers
  for (i=0; i<containerList.no(); i++) {
    FXXMenuContainer *w=containerList[i];
    if (w)
      w->disable();
  }

}



void FXXAction::create() {
  FXint i;

  // create in all menu commands
  for (i=0; i<commandList.no(); i++) {
    FXWindow *w=commandList[i];
    if (w)
      w->create();
  }

  // create in all menu checks
  for (i=0; i<menuCheckList.no(); i++) {
    FXWindow *w=menuCheckList[i];
    if (w)
      w->create();
  }

  // create in all tool buttons
  for (i=0; i<toolButtonList.no(); i++) {
    FXWindow *w=toolButtonList[i];
    if (w)
      w->create();
  }

  // create in all check buttons
  for (i=0; i<checkButtonList.no(); i++) {
    FXWindow *w=checkButtonList[i];
    if (w)
      w->create();
  }

  // enable in all menu containers
  for (i=0; i<containerList.no(); i++) {
    FXXMenuContainer *w=containerList[i];
    if (w)
      w->create();
  }
}



void FXXAction::removeFromAll() {
  FXint i;

  // remove from all menu commands
  for (i=0; i<commandList.no(); i++) {
    FXXActionMenuCommand *w=commandList[i];
    if (w) {
      w->action=NULL;
      delete w;
    }
  }
  commandList.clear();

  // remove from all tool buttons
  for (i=0; i<toolButtonList.no(); i++) {
    FXXActionToolButton *w=toolButtonList[i];
    if (w) {
      w->action=NULL;
      delete w;
    }
  }
  toolButtonList.clear();

  // remove from all menu containers
  for (i=0; i<containerList.no(); i++) {
    FXXMenuContainer *w=containerList[i];
    if (w) {
      w->action=NULL;
      delete w;
    }
  }
  containerList.clear();

}



void FXXAction::addToMenu(FXMenuPane* p,
                          FXuint opts,
                          FXbool withText, FXbool withIcon) {
  FXMenuCommand *cmd;

  cmd=new FXXActionMenuCommand(this,
                               p,
                               withText?text:FXString(""),
                               withIcon?iconSet->getSmallIcon():NULL,
                               target, selector);
  cmd->setHelpText(helpText);
  cmd->setTipText(tipText);
  commandList.append(cmd);
}



void FXXAction::addToMenu(FXXMenuContainer *p,
                          FXuint opts,
                          FXbool withText, FXbool withIcon) {
  FXXMenuContainer *cmd;

  cmd=FXXMenuContainer::createMenuCommand(p, this, withText, withIcon, opts);
  containerList.append(cmd);
}



void FXXAction::addToToolBar(FXToolBar *p,
                             FXuint opts,
                             FXbool withText, FXbool withIcon) {
  FXButton *b;
  FXFont *fnt;

  b=new FXXActionToolButton(this, p,
                            withText?text:FXString(""),
                            withIcon?iconSet->getMediumIcon():NULL,
                            target, selector,
                            BUTTON_TOOLBAR|
                            FRAME_RAISED|FRAME_THICK|
                            JUSTIFY_CENTER_X|
                            ICON_ABOVE_TEXT);
  b->setHelpText(helpText);
  b->setTipText(tipText);

  fnt=b->getFont();
  if (fnt) {
    FXFontDesc descr;
    FXFont *newFnt;

    fnt->getFontDesc(descr);
    fprintf(stderr, "Font size: %d\n", descr.size);
    descr.size=60;
    newFnt=new FXFont(fnt->getApp(), descr);
    b->setFont(newFnt);
  }
  toolButtonList.append(b);
}



void FXXAction::addToMenuCheckable(FXMenuPane* p, FXuint opts) {
  FXMenuCommand *cmd;

  cmd=new FXXActionMenuCheck(this,
                             p,
                             text,
                             target, selector);
  cmd->setHelpText(helpText);
  cmd->setTipText(tipText);
  menuCheckList.append(cmd);
}



void FXXAction::addToToolBarCheckable(FXToolBar *p, FXuint opts) {
  FXCheckButton *b;
  FXFont *fnt;

  b=new FXXActionCheckButton(this, p,
			     text,
			     target, selector,
			     BUTTON_TOOLBAR|
			     FRAME_RAISED|FRAME_THICK|
			     JUSTIFY_CENTER_X);
  b->setHelpText(helpText);
  b->setTipText(tipText);

  fnt=b->getFont();
  if (fnt) {
    FXFontDesc descr;
    FXFont *newFnt;

    fnt->getFontDesc(descr);
    fprintf(stderr, "Font size: %d\n", descr.size);
    descr.size=60;
    newFnt=new FXFont(fnt->getApp(), descr);
    b->setFont(newFnt);
  }
  checkButtonList.append(b);
}



void FXXAction::removeFrom(FXWindow *p) {
  FXint i;

  // remove from menu command list
  for (i=0; i<commandList.no(); i++) {
    FXWindow *w;

    w=commandList[i];
    while(w) {
      if (w==p) {
        commandList.erase(i);
        i--;
        break;
      }
      w=w->getParent();
    }
  }


  // remove from menu check list
  for (i=0; i<menuCheckList.no(); i++) {
    FXWindow *w;

    w=menuCheckList[i];
    while(w) {
      if (w==p) {
        menuCheckList.erase(i);
        i--;
        break;
      }
      w=w->getParent();
    }
  }

  // remove from tool button list
  for (i=0; i<toolButtonList.no(); i++) {
    FXWindow *w;

    w=toolButtonList[i];
    while(w) {
      if (w==p) {
        toolButtonList.erase(i);
        i--;
        break;
      }
      w=w->getParent();
    }
  }

  // remove from check button list
  for (i=0; i<checkButtonList.no(); i++) {
    FXWindow *w;

    w=checkButtonList[i];
    while(w) {
      if (w==p) {
        checkButtonList.erase(i);
        i--;
        break;
      }
      w=w->getParent();
    }
  }
}




void FXXAction::removeMenuCommand(FXXActionMenuCommand *w) {
  FXint i;

  // remove from menu command list
  for (i=0; i<commandList.no(); i++) {
    if (commandList[i]==w) {
      commandList.erase(i);
      break;
    }
  }
}



void FXXAction::removeMenuCheck(FXXActionMenuCheck *w) {
  FXint i;

  // remove from menu command list
  for (i=0; i<menuCheckList.no(); i++) {
    if (menuCheckList[i]==w) {
      menuCheckList.erase(i);
      break;
    }
  }
}



void FXXAction::removeToolButton(FXXActionToolButton *w) {
  FXint i;

  // remove from tool button list
  for (i=0; i<toolButtonList.no(); i++) {
    if (toolButtonList[i]==w) {
      toolButtonList.erase(i);
      break;
    }
  }
}



void FXXAction::removeCheckButton(FXXActionCheckButton *w) {
  FXint i;

  // remove from tool button list
  for (i=0; i<checkButtonList.no(); i++) {
    if (checkButtonList[i]==w) {
      checkButtonList.erase(i);
      break;
    }
  }
}



void FXXAction::removeMenuContainer(FXXMenuContainer *w) {
  FXint i;

  // remove from menu command list
  for (i=0; i<containerList.no(); i++) {
    if (containerList[i]==w) {
      containerList.erase(i);
      break;
    }
  }
}






void FXXAction::setCheck(FXbool s) {
  FXint i;

  m_check=s;

  for (i=0; i<menuCheckList.no(); i++)
    menuCheckList[i]->setCheck(m_check);

  for (i=0; i<checkButtonList.no(); i++)
    checkButtonList[i]->setCheck(m_check);
}





