/***************************************************************************
 * This file is part of the project AqFoxExt.                              *
 * AqFoxExt (c) by 2011 Martin Preuss, all rights reserved.                *
 *                                                                         *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fxx_comboedit.hpp"

#include <fxkeys.h>



// map
FXDEFMAP(FXXComboEdit) FXXComboEditMap[]={
  FXMAPFUNC(SEL_CHANGED,FXComboBox::ID_TEXT,FXXComboEdit::onTextChanged),
  FXMAPFUNC(SEL_COMMAND,FXComboBox::ID_TEXT,FXXComboEdit::onTextCommand),
  FXMAPFUNC(SEL_KEYPRESS,FXComboBox::ID_TEXT,FXXComboEdit::onTextKeyPress),
  FXMAPFUNC(SEL_FOCUS_UP,0,FXXComboEdit::onFocusUp),
  FXMAPFUNC(SEL_FOCUS_DOWN,0,FXXComboEdit::onFocusDown),
  FXMAPFUNC(SEL_MOUSEWHEEL,FXComboBox::ID_TEXT,FXXComboEdit::onMouseWheel),
  FXMAPFUNC(SEL_FOCUSIN,0,FXXComboEdit::onFocusIn),
  FXMAPFUNC(SEL_FOCUS_SELF,0,FXXComboEdit::onFocusSelf),
};


// Object implementation
FXIMPLEMENT(FXXComboEdit,THEMECOMBOBOX,FXXComboEditMap,ARRAYNUMBER(FXXComboEditMap))



FXXComboEdit::FXXComboEdit(FXComposite *p, FXint cols,
			   FXObject* tgt,
			   FXSelector sel,
			   FXuint opts,
			   FXint x, FXint y, FXint w, FXint h,
			   FXint pl, FXint pr, FXint pt, FXint pb)
:THEMECOMBOBOX(p, cols, tgt, sel, opts, x, y, w, h, pl, pr, pt, pb)
,m_neededWidth(0)
{
  field->setTextStyle(TEXTFIELD_ENTER_ONLY);
}



FXXComboEdit::~FXXComboEdit() {
}



void FXXComboEdit::addEntry(const char *s) {
  FXFont* fnt;
  int w;
  FXString str;

  FXASSERT(s);
  str=fromAscii(s);
  _entries.push_back(str);

  fnt=getFont();
  /* get font width, also take into account scrolbar and borders */
  w=fnt->getTextWidth(str)+20;
  if (w>m_neededWidth)
    m_neededWidth=w;
}



void FXXComboEdit::_populateList(FXbool adjustPaneState) {
  FXString txt=getText().lower();
  int count=0;
  int numvis;

  list->clearItems();
  if (!txt.empty()) {
    std::list<FXString>::iterator it;

    for (it=_entries.begin(); it!=_entries.end(); it++) {
      FXString str=(*it);

      if (str.lower().contains(txt)!=0) {
	list->appendItem((*it), NULL, NULL);
	count++;
      }
    }
  }
  else {
    std::list<FXString>::iterator it;

    /* empty text, so all entries match */
    for (it=_entries.begin(); it!=_entries.end(); it++) {
      list->appendItem((*it), NULL, NULL);
      count++;
    }
  }

  if (count>10)
    numvis=10;
  else
    numvis=count;
  if (getNumVisible()!=numvis) {
    setNumVisible(numvis);
    layout();
  }

  if (adjustPaneState) {
    if (count) {
      if (!isPaneShown())
        button->handle(this,FXSEL(SEL_COMMAND,ID_POST),NULL);
    }
    else {
      if (isPaneShown())
        button->handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
    }
  }
}


long FXXComboEdit::onTextChanged(FXObject *obj, FXSelector sel, void *ptr) {
  _populateList(TRUE);
  return THEMECOMBOBOX::onTextChanged(obj, sel, ptr);
}



long FXXComboEdit::onTextCommand(FXObject *obj, FXSelector sel, void *ptr) {
  FXTRACE((150,"%s::onTextCommand %p\n",getClassName(),this));
  if (isPaneShown()) {
    FXint index=list->getCurrentItem();

    /* take over text from list */
    setText(getItemText(index));
    /* hide pane */
    button->handle(this, FXSEL(SEL_COMMAND, ID_UNPOST), NULL);
    return THEMECOMBOBOX::onTextCommand(obj, sel, ptr);
  }
  else {
    /* pane is not shown */
    return THEMECOMBOBOX::onTextCommand(obj, sel, ptr);
  }
  return 0;
}




long FXXComboEdit::onTextKeyPress(FXObject *obj, FXSelector sel, void *ptr) {
  FXEvent* event=(FXEvent*)ptr;

  switch(event->code) {
  case KEY_Escape:
    if (isPaneShown()) {
      button->handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
      return 1;
    }
    else {
      if (target && target->handle(this, FXSEL(SEL_DESELECTED, message), NULL))
        return 1;
    }

  default:
    break;
  }

  return THEMECOMBOBOX::handle(obj, sel, ptr);
}



long FXXComboEdit::onFocusSelf(FXObject* sender,FXSelector sel,void* ptr){
  FXTRACE((150,"%s::onFocusSelf %p\n",getClassName(),this));
  THEMECOMBOBOX::onFocusSelf(sender, sel, ptr);
  return field->handle(sender,FXSEL(SEL_FOCUS_SELF,0),ptr);
}



// Select upper item
long FXXComboEdit::onFocusUp(FXObject*,FXSelector,void*){
  FXTRACE((150,"%s::onFocusUp %p\n",getClassName(),this));
  if (isEnabled()){
    FXint index=getCurrentItem();
    if(index<0) index=getNumItems()-1;
    else if(0<index) index--;
    if(0<=index && index<getNumItems()) {
      setCurrentItem(index,FALSE);
    }
    return 1;
  }
  return 0;
}


// Select lower item
long FXXComboEdit::onFocusDown(FXObject*,FXSelector,void*){
  FXTRACE((150,"%s::onFocusDown %p\n",getClassName(),this));
  if (isEnabled()){
    FXint index=getCurrentItem();
    if(index<0) index=0;
    else if(index<getNumItems()-1) index++;
    if(0<=index && index<getNumItems()){
      setCurrentItem(index,FALSE);
    }
    return 1;
  }
  return 0;
}


// Mouse wheel
long FXXComboEdit::onMouseWheel(FXObject*,FXSelector,void* ptr){
  FXEvent* event=(FXEvent*)ptr;

  if (isEnabled()){
    FXint index=getCurrentItem();
    if(event->code<0){
      if(index<0) index=0;
      else if(index<getNumItems()-1) index++;
    }
    else if(event->code>0){
      if(index<0) index=getNumItems()-1;
      else if(0<index) index--;
    }
    if(0<=index && index<getNumItems()){
      setCurrentItem(index,FALSE);
    }
    return 1;
  }
  return 0;
}



long FXXComboEdit::onFocusIn(FXObject *sender, FXSelector sel,void *ptr){
  THEMECOMBOBOX::onFocusIn(sender, sel, ptr);
  field->setFocus();
  return 1;
}



void FXXComboEdit::setText(const FXString& text) {
  THEMECOMBOBOX::setText(text);
  _populateList(FALSE);
}



FXString FXXComboEdit::getText() {
  return field->getText();
}



void FXXComboEdit::selectAll() {
  field->selectAll();
}



void FXXComboEdit::reloadList() {
  _populateList(FALSE);
}



void FXXComboEdit::layout(){
  int w;

  THEMECOMBOBOX::layout();
  w=pane->getWidth();
  if (w<m_neededWidth)
    pane->resize(m_neededWidth, pane->getDefaultHeight());
}








